import NfHttp from "../index";
import NfStore from "../../vuex";
import NfAxios from "./index";
import UtilNotEmpty from "../../../utils/js/UtilNotEmpty";
import NfRouter from "../../router";
import UtilLocalStorage from "../../../utils/js/UtilLocalStorage";
import {LOCAL_STORAGE_KEY, CLIENT_INIT} from "../../../utils/const/NfConst";

export default class RefreshToken {
  constructor() {
    this.refreshTokenFlag = false;
    this.cacheRequests = [];
  };

  /**
   * 请求新的token
   */
  api_refreshToken() {
    let client = CLIENT_INIT.client;
    if (client) {
      let params = {
        client_id: client.client_id,
        client_secret: client.client_secret,
        grant_type: "refresh_token",
        refresh_token: ""
      };
      //统一携带token
      let {refresh_token} = NfStore.instance.getters.loginToken;
      if (UtilNotEmpty.obj(refresh_token)) {
        params.refresh_token = refresh_token;
      }
      return NfHttp.instance.post(CLIENT_INIT.base_url.refresh_token, null, params);
    }
  }

  /**
   * 根据请求config，缓存token过期的请求
   * @param config
   */
  createCacheRequest(config) {
    // 将resolve放进队列，用一个匿名函数的形式来保存，接收access_token参数,执行时调用函数传入access_token,发起请求。
    this.cacheRequests.push(() => {
      NfAxios.instance(config);
    })
  }


  /**
   * 刷新token后
   */
  refreshTokenAndSendRequests(config) {
    //token过期，刷新最新的token
    if (!this.refreshTokenFlag) {
      //防止多个请求同时去刷新token
      this.refreshTokenFlag = true;
      this.api_refreshToken().then(loginToken => {
        //放开刷新token锁 请求
        this.refreshTokenFlag = false;
        let {access_token} = loginToken;
        NfStore.instance.commit('setLoginToken', loginToken);
        if (access_token) {
          //将当前请求放入队列中
          this.cacheRequests.unshift(() => {
            NfAxios.instance(config);
          });
          //将缓存的请求队列依次重新请求
          this.cacheRequests.forEach(cacheRequestFunc => {
            setTimeout(cacheRequestFunc(), 1000);
          });
        } else {
          //未获取到新的token，返回登录页
          this.returnLoginPage();
        }
        //清空缓存的请求
        this.cacheRequests = [];
      }).catch(res => {
        //刷新token出错，返回登录页
        console.error(res);
        this.returnLoginPage();
      }).finally(() => {
        //最终关闭刷新token标记
        this.refreshTokenFlag = false;
      });
    } else {
      // 正在刷新token，将返回一个未执行resolve的promise
      this.createCacheRequest(config);
    }
  }

  /**
   * 清除所有状态，返回登录页
   */
  returnLoginPage() {
    let obj_store = UtilLocalStorage.getItemObj(LOCAL_STORAGE_KEY.VUEX_STORE_INIT);
    if (UtilNotEmpty.obj(obj_store)) {
      NfStore.instance.replaceState(Object.assign(NfStore.instance.state, obj_store));
    }
    NfRouter.instance.push(NfRouter.nfPathLogin);
  }
}
