import NfAxios from "./axios/index";
import UtilNotEmpty from "../../utils/js/UtilNotEmpty";
import UtilEmpty from "../../utils/js/UtilEmpty";

export default class NfHttp {
  constructor(nfAxios) {
    this.nfAxios = nfAxios;
  }

  static init(config) {
    if (!this.instance) {
      this.instance = new NfHttp(NfAxios.init(config));
    }
    return this.instance;
  }

  get(url, params) {
    if (!this.nfAxios) {
      throw "未初始化axios实例，无法发出请求";
    }
    return this.nfAxios.get(url, params);
  };

  post(url, data, params = null) {
    if (!this.nfAxios) {
      throw "未初始化axios实例，无法发出请求";
    }
    if (UtilNotEmpty.obj(params)) {
      return this.nfAxios.post(url, data, {params: params});
    } else {
      return this.nfAxios.post(url, data ? data : {});
    }
  };

  upload(url, data) {
    return this.nfAxios(
      {
        method: "post",
        url: url,
        data: data,
        headers: {'Content-Type': 'multipart/form-data'},
      }
    );
  }

  download(url, data) {
    if (!this.nfAxios) {
      throw "未初始化axios实例，无法发出请求";
    }
    this.nfAxios({method: 'post', url: url, data: data, responseType: 'blob'})
      .then((res) => {
        const content = res.data;
        const disposition = res.headers['content-disposition'];
        const blob = new Blob([content]);
        if (UtilEmpty.string(disposition)) {
          throw "下载流header参数未设置content-disposition";
        }
        const fileName = disposition.replace(/\w+;filename=(.*)/, '$1')
        if (UtilEmpty.string(fileName)) {
          throw "header content-disposition 参数缺少filename";
        }
        if ('download' in document.createElement('a')) {
          // 非IE下载
          const emlink = document.createElement('a');
          emlink.download = fileName;
          emlink.style.display = 'none';
          emlink.href = URL.createObjectURL(blob);
          document.body.appendChild(emlink);
          emlink.click();
          // 释放URL 对象
          URL.revokeObjectURL(emlink.href);
          document.body.removeChild(emlink);
        } else {
          // IE10+下载
          navigator.msSaveBlob(blob, fileName)
        }
      })
      .catch(err => {
        throw "下载失败" + JSON.stringify(err)
      });
  };
}
