import UtilEmpty from '../../../utils/js/UtilEmpty'
import UtilLoading from "../../../utils/js/UtilLoading";
import UtilMessage from "../../../utils/js/UtilMessage";
import UtilNotEmpty from "../../../utils/js/UtilNotEmpty";
import {TABS_KEY, PERMISSION_KEY, MENU_KEY} from "../../../utils/const/NfConst";

const state = {
  //右侧顶部标签数组
  tabs: [],
  //当前活动标签页
  curTabId: "",
  homeTab: {},
  //权限池
  perCodePool: {},
};
// 设置变化
const mutations = {
  /**
   * 打开tab页
   */
  ['openTab'](state, menu) {
    if (UtilEmpty.obj(menu) || UtilEmpty.string(menu[TABS_KEY.component])) {
      return;
    }
    let menuId = UtilEmpty.string(menu[MENU_KEY.id]) ? "" : menu[MENU_KEY.id];
    let tabId = menu[TABS_KEY.component] + '_' + menuId;
    let isOpen = false;
    for (let i = 0; i < state.tabs.length; i++) {
      let nowTab = state.tabs[i];
      if (nowTab[TABS_KEY.id] === tabId) {
        //标签已经打开
        state.curTabId = tabId;
        isOpen = true;
        break;
      }
    }
    if (isOpen === false) {
      let curTab = {};
      //show字段用来实现手动刷新标签页
      curTab.show = true;
      curTab[TABS_KEY.id] = tabId;
      curTab[TABS_KEY.menu] = menu;
      curTab[TABS_KEY.label] = menu[MENU_KEY.label];
      curTab[TABS_KEY.component] = menu[MENU_KEY.tab];
      curTab[TABS_KEY.params] = menu[MENU_KEY.params];
      state.tabs.push(curTab);
      state.curTabId = tabId;
      this.commit('appendPerCodes', menu[MENU_KEY.permissions]);
    }
  },
  ['appendPerCodes'](state, permissions) {
    if (UtilNotEmpty.array(permissions)) {
      for (let i = 0; i < permissions.length; i++) {
        let id = permissions[i][PERMISSION_KEY.id];
        let code = permissions[i][PERMISSION_KEY.code];
        if (UtilNotEmpty.string(id) && UtilNotEmpty.string(code)) {
          state.perCodePool[code] = id;
        }
      }
    }
  },
  /**
   * 点击右侧tab页重新打开:触发tab的活动页自动切换
   */
  ['reloadTab'](state, tabId) {
    state.curTabId = tabId;
  },
  /**
   * 关闭当前活动tab
   */
  ['removeTab'](state, tabId) {
    for (let j = 0; j < state.tabs.length; j++) {
      if (state.tabs[j][TABS_KEY.id] === tabId) {
        if ((j - 1) >= 0) {
          state.curTabId = state.tabs[j - 1][TABS_KEY.id];
        } else if ((j + 1) < state.tabs.length) {
          state.curTabId = state.tabs[j + 1][TABS_KEY.id];
        }
        state.tabs.splice(j, 1);
        break;
      }
    }
    if (state.tabs.length === 0) {
      state.curTabId = state.homeTab[TABS_KEY.id];
    }
  },
  /**
   * 关闭左侧tab
   */
  ['removeLeftTab'](state, tabId) {
    for (let j = 0; j < state.tabs.length; j++) {
      if (state.tabs[j][TABS_KEY.id] === tabId) {
        let curIndex = j;
        if (curIndex > 0) {
          state.tabs.splice(0, curIndex);
        }
        break;
      }
    }
  },
  /**
   * 关闭右侧tab
   */
  ['removeRightTab'](state, tabId) {
    for (let j = 0; j < state.tabs.length; j++) {
      if (state.tabs[j][TABS_KEY.id] === tabId) {
        let curIndex = j;
        if (curIndex < (state.tabs.length - 1)) {
          state.tabs.splice(curIndex + 1, state.tabs.length - curIndex + 1);
        }
        break;
      }
    }
  },
  /**
   * 关闭关联tab
   * @param state
   * @param relativeId
   */
  ['removeRelativeTab'](state, relativeId) {
    for (let j = 0; j < state.tabs.length; j++) {
      let tabId = state.tabs[j][TABS_KEY.id];
      if (tabId.indexOf(relativeId) !== -1) {
        this.commit('removeTab', tabId);
      }
    }
  },
  /**
   * 关闭全部tab
   */
  ['removeAllTab'](state) {
    state.tabs = [];
    state.curTabId = state.homeTab[TABS_KEY.id];
  },
  /**
   * 关闭其他tab
   */
  ['removeOtherTab'](state, tabId) {
    let curTab = null;
    for (let j = 0; j < state.tabs.length; j++) {
      if (state.tabs[j][TABS_KEY.id] === tabId) {
        curTab = state.tabs[j];
        break;
      }
    }
    if (null !== curTab) {
      state.tabs = [curTab];
    }
  },
  /**
   * 设置homeTab
   * @param state
   * @param home
   */
  ['setHomeTab'](state, home) {
    if (UtilEmpty.obj(state.homeTab)) {
      state.homeTab = home;
      state.curTabId = home[TABS_KEY.id];
      let homeMenu = home[TABS_KEY.menu];
      if (UtilNotEmpty.obj(homeMenu)) {
        this.commit('appendPerCodes', homeMenu[MENU_KEY.permissions]);
      }
    }
  },
  ['refreshTabData'](state, tabId) {
    if (state.curTabId !== tabId) {
      UtilMessage.warning("请打开该标签页后再刷新");
      return;
    }
    let curTab = null;
    for (let j = 0; j < state.tabs.length; j++) {
      if (state.tabs[j][TABS_KEY.id] === tabId) {
        curTab = state.tabs[j];
        break;
      }
    }
    if (null != curTab) {
      curTab.show = false;
      UtilLoading.showLoading();
      setTimeout(() => {
        curTab.show = true;
        UtilLoading.hideLoading();
      }, 500)
    }
  }
};
const actions = {
  openTab(context, menu) {
    context.commit('openTab', menu);
  },
  reloadTab(context, tabId) {
    context.commit('reloadTab', tabId);
  },
  removeTab(context, tabId) {
    context.commit('removeTab', tabId);
  },
  removeRelativeTab(context, relativeId) {
    context.commit('removeRelativeTab', relativeId);
  },
  removeLeftTab(context, tabId) {
    context.commit('removeLeftTab', tabId);
  },
  removeRightTab(context, tabId) {
    context.commit('removeRightTab', tabId);
  },
  removeAllTab(context) {
    context.commit('removeAllTab');
  },
  removeOtherTab(context, tabId) {
    context.commit('removeOtherTab', tabId);
  },
  setHomeTab(context, homeTab) {
    context.commit('setHomeTab', homeTab);
  },
  refreshTabData(context, tabId) {
    context.commit('refreshTabData', tabId);
  },
  appendPerCodes(context, permissions) {
    context.commit('appendPerCodes', permissions);
  }
};
const getters = {
  tabs: state => {
    return state.tabs;
  },
  curTabId: state => {
    return state.curTabId;
  },
  perCodePool: state => {
    return state.perCodePool;
  }
};
export default {
  state,
  mutations,
  actions,
  getters,
}
